/*************************************************************
Module name: Sum.CPP
Notices: Copyright (c) 1994 Jeffrey Richter
*************************************************************/


#define STRICT
#define UNICODE
#define _UNICODE
#include <windows.h>
#include <windowsx.h>
#include <limits.h>

#include "Resource.h"


/////////////////////////////////////////////////////////////


// An example of calling Sum for uNum = 0 to 9
// uNum: 0 1 2 3  4  5  6  7	8  9 ...
// Sum:  0 1 3 6 10 15 21 28 36 45 ...
UINT Sum (UINT uNum) {

	if (uNum == 0) 
		return(0);

	// Call Sum recursively.
	return(uNum + Sum(uNum - 1));
}


/////////////////////////////////////////////////////////////


DWORD FilterFunc (DWORD dwExceptionCode) {

	// Assume that we do not know how to handle the exception
	// by teling the system to continue to search for an SEH
	// handler.
	DWORD dwRet = EXCEPTION_CONTINUE_SEARCH;

	if (dwExceptionCode == STATUS_STACK_OVERFLOW) {
		// If the exception raised is a stack overflow,
		// then we do know how to handle it.
		dwRet = EXCEPTION_EXECUTE_HANDLER;
	}

	return(dwRet);
}


/////////////////////////////////////////////////////////////


// The separate thread that is responsible for calculating the
// Sum.  I use a separate thread for the following reasons:
// 1. A separate thread gets it's own 1 MB of stack space.
// 2. A thread can only be notified of a stack overflow once.
// 3. The stack's storage is freed when the thread exits.
DWORD WINAPI SumThreadFunc (PVOID p) {

	// The parmeter, p, is really a UINT containing 
	// the number of numbers to sum.
	UINT uSumNum = (UINT) p;

	// uSum contains the summation of the numbers
	// from 0 to uSumNum.  If the sum cannot be calculated,
	// a sum of UINT_MAX is returned.
	UINT uSum = UINT_MAX;

	__try {
		// To catch the stack overflow exception, we must 
		// execute the Sum function while inside an SEH block.
		uSum = Sum(uSumNum);	
	}
	__except (FilterFunc(GetExceptionCode())) {
		// If we get in here, it's because we have trapped
		// a stack overflow.  We can now do whatever is 
		// necessary to gracefully continue execution.
		// This sample application has nothing to do so
		// no code is placed in this exception handler block.
	}

	// The thread's exit code is the Sum of the first uSumNum 
	// numbers or UINT_MAX if a stack overflow occurred.
	return(uSum);
}


/////////////////////////////////////////////////////////////


BOOL Dlg_OnInitDialog (HWND hwnd, HWND hwndFocus, 
   LPARAM lParam) {

   // Associate an icon with the dialog box.
   SetClassLong(hwnd, GCL_HICON, (LONG)
      LoadIcon((HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE),
      __TEXT("Summation")));

   return(TRUE);
}


/////////////////////////////////////////////////////////////


void Dlg_OnCommand (HWND hwnd, int id, 
   HWND hwndCtl, UINT codeNotify) {

	UINT uSumNum, uSum;
	BOOL fTranslated;
	DWORD dwThreadId;
	HANDLE hThread;
	   
   switch (id) {
		case IDC_CALC:
			// Get the number of numbers the user wants to sum.
			uSumNum = GetDlgItemInt(hwnd, IDC_SUMNUM,
				&fTranslated, FALSE);

			// Create a thread (with it's own stack) that is
			// responsible for performing the summation.
			hThread = CreateThread(NULL, 0, SumThreadFunc, 
				(PVOID) uSumNum, 0, &dwThreadId);

			// The thread's exit code is the resulting summation.
			
			// We must first wait for the thread to terminate.
			WaitForSingleObject(hThread, INFINITE);

			// Now, we can get the thread's exit code.
			GetExitCodeThread(hThread, (PDWORD) &uSum);

			// Finally, we close the handle to the thread so
			// that the system can destroy the thread object.
			CloseHandle(hThread);

			// Update the dialog box so show the result.
			if (uSum == UINT_MAX) {
				// If the thread's exit code is UINT_MAX, a stack 
				// overflow occurred.  Update the dialog box 
				// and display a message box.
				SetDlgItemText(hwnd, IDC_ANSWER, __TEXT("Error"));
				MessageBox(GetFocus(), 
					__TEXT("Summation could not be calculated"), 
					NULL, MB_OK);
			} else {
				// The summation was calculated successfully; 
				// update the dialog box.
				SetDlgItemInt(hwnd, IDC_ANSWER, uSum, FALSE);
			}

			break;
        
      case IDCANCEL:
         EndDialog(hwnd, id);
         break;
   }
}

/////////////////////////////////////////////////////////////


BOOL CALLBACK Dlg_Proc (HWND hDlg, UINT uMsg, 
   WPARAM wParam, LPARAM lParam) {

   BOOL fProcessed = TRUE;

   switch (uMsg) {
      HANDLE_MSG(hDlg, WM_INITDIALOG, Dlg_OnInitDialog);
      HANDLE_MSG(hDlg, WM_COMMAND, Dlg_OnCommand);

      default:
         fProcessed = FALSE;
         break;
   }

   return(fProcessed);
}


/////////////////////////////////////////////////////////////


int APIENTRY WinMain (HINSTANCE hInstance, 
   HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow) {
   
   DialogBox(hInstance, MAKEINTRESOURCE(DLG_SUMMATION), 
      NULL, Dlg_Proc);

   return(0);
}


//////////////////////// End Of File ////////////////////////
